@@ -26,12 +26,16 @@ from typing import Union, Dict, List
 from datetime import datetime
 import gzip
 import json
-import os
 import numpy
 import base64
 from io import BytesIO
-from roguewave import to_datetime_utc
+from roguewave import to_datetime_utc, filecache
 from xarray import Dataset, open_dataset, DataArray
+import boto3
+from botocore.errorfactory import ClientError
+import tempfile
+import pickle
+import os
 
 
 _UNION = Union[
@@ -130,7 +134,7 @@ class NumpyEncoder(json.JSONEncoder):
 def object_hook(dictionary: dict):
     if "__class__" in dictionary:
         if dictionary["__class__"] == "DataFrame":
-            #
+            # This is a mess- needs to be refactored.
             df = read_json(dictionary["data"])
             if "timestamp" in df:
                 df["timestamp"] = df["timestamp"].apply(lambda x: x.tz_localize("utc"))
@@ -178,51 +182,100 @@ def object_hook(dictionary: dict):
         return dictionary
 
 
-def load(filename: str) -> _UNION:
+def load(filename: str, force_redownload_if_remote=False, filetype="roguewave"):
     """
     Load spectral data as saved by "save_spectrum" from the given file and
     return a (nested) object. The precise format of the output depends on what
     was saved.
 
-    :param filename: path to file to load.
+    :param filename: path to file to load. If an s3 uri is given (of the form s3://bucket/key) the remote file is
+        downloaded and cached locally. Future requests of the same uri are retrieved from the cache.
+
+    :param force_redownload_if_remote: If s3 file is cached force a refresh with the remote resource.
+
+    :param filetype: ['roguewave', 'pickle', 'netcdf']. Type of file to be loaded.
+
     :return:
         - Data in the same form it was saved.
     """
 
-    with gzip.open(filename, "rb") as file:
-        data = file.read().decode("utf-8")
-    return json.loads(data, object_hook=object_hook)
+    if filename.startswith("s3://"):
+        # Get the file from s3 and cache locally
+        if force_redownload_if_remote:
+            filecache.delete_files(filename, error_if_not_in_cache=False)
+        filename = filecache.filepaths([filename])[0]
+
+    if filetype == "roguewave":
+        with gzip.open(filename, "rb") as file:
+            data = file.read().decode("utf-8")
+        return json.loads(data, object_hook=object_hook)
+    elif filetype == "pickle":
+        with open(filename, "rb") as file:
+            data = pickle.load(file)
+        return data
+    elif filetype == "netcdf":
+        return open_dataset(filename)
 
 
 def save(
-    _input: _UNION, filename: str, format="json", separate_spotters_into_files=False
+    _input: _UNION, filename: str, overwrite=True, s3_overwrite=False, use_pickle=False
 ):
     """
-    Save spectral data, possible in nested form as returned by the spectral
-    partition/reconstruction/etc. functions. Data is saved in JSON form
-    compressed with gzip.
+    Save roguewave data in JSON form compressed with gzip.
 
     :param _input:
         - Data containing python primitives (dict/list) or any of the RogueWave
           classes.
 
-    :param filename: path to save data.
+    :param filename: path to save data. If an s3 uri is given (of the form s3://bucket/key) the  file is
+        saved to s3. If a local file already exists with the same name the file is overwritten if overwrite = True
+        (default), otherwise an error is raised. If an s3 object with the same uri exists we raise an error (unless
+        s3_overwrite = True).
+
+    :param overwrite: Default True. If False an error is raised if a file with the same name already exists. By default
+        we simply overwrite under the assumption we are aware of local context.
+
+    :param s3_overwrite: Default False. If False an error is raised if an object with the same uri already exists on s3.
+        By default we raise an error as we may clash with keys from others.
+
+    :param use_pickle: Default False. Use the pickle protocol to save the object
+
     :return: None
     """
 
-    def write(filename, _input, format):
-        if format == "json":
+    def write(filename, _input):
+        if use_pickle:
+            with open(filename, "wb") as file:
+                pickle.dump(_input, file)
+        else:
             with gzip.open(filename, "wt") as file:
                 json.dump(_input, file, cls=NumpyEncoder)
-                # file.write(data.encode('utf-8'))
-        else:
-            raise Exception("Unknown output format")
 
-    if separate_spotters_into_files:
-        os.makedirs(filename, exist_ok=True)
-        for key in _input:
-            name = os.path.join(filename, key)
-            write(name, _input[key], format)
+    if filename.startswith("s3://"):
+        with tempfile.TemporaryFile() as temp_file:
+            write(temp_file, _input)
+            temp_file.seek(0)
+
+            bucket, key = filename.removeprefix("s3://").split("/", 1)
+            s3 = boto3.client("s3")
+
+            try:
+                s3.head_object(Bucket=bucket, Key=key)
+                if not s3_overwrite:
+                    raise FileExistsError(
+                        f"Key {key} already exists in bucket {bucket}. To overwrite the file (if"
+                        f"desired) set s3_overwrite=True"
+                    )
+            except ClientError:
+                # Not found
+                pass
+
+            s3.upload_fileobj(temp_file, bucket, key)
 
     else:
-        write(filename, _input, format)
+        if os.path.exists(filename) and not overwrite:
+            raise FileExistsError(
+                f"Key {filename} already exists. To overwrite the file (if"
+                f"desired) set overwrite=True"
+            )
+        write(filename, _input)
