@@ -14,9 +14,22 @@ from roguewave.wavespectra.estimators import (
     Estimators,
 )
 from typing import Iterator, Hashable, TypeVar, Union, List, Literal, Mapping
-from xarray import Dataset, DataArray, open_dataset, concat, ones_like, where
+from xarray import (
+    Dataset,
+    DataArray,
+    open_dataset,
+    concat,
+    ones_like,
+    where,
+    zeros_like,
+)
 from xarray.core.coordinates import DatasetCoordinates
 from warnings import warn
+from roguewave.wavespectra._tools import (
+    numba_fill_zeros_or_nan_in_tail,
+    spline_peak_frequency,
+    _cdf_interpolate,
+)
 
 NAME_F: Literal["frequency"] = "frequency"
 NAME_D: Literal["direction"] = "direction"
@@ -43,6 +56,7 @@ NAMES_1D = (
     NAME_DEPTH,
 )
 SPECTRAL_VARS = (NAME_E, NAME_a1, NAME_b1, NAME_a2, NAME_b2)
+SPECTRAL_MOMENTS = (NAME_a1, NAME_b1, NAME_a2, NAME_b2)
 SPECTRAL_DIMS = (NAME_F, NAME_D)
 SPACE_TIME_DIMS = (NAME_T, NAME_LON, NAME_LAT)
 
@@ -209,6 +223,11 @@ class WaveSpectrum(DatasetWrapper):
             coords={NAME_F: self.frequency},
         )
 
+    def fillna(self, value=0.0):
+        for variable in SPECTRAL_VARS:
+            if variable in self.dataset:
+                self.dataset[variable] = self.dataset[variable].fillna(value)
+
     def is_invalid(self) -> DataArray:
         return self.variance_density.isnull().all(dim=self.dims_spectral)
 
@@ -259,7 +278,11 @@ class WaveSpectrum(DatasetWrapper):
         dims = self.dims_space_time
         coords = self.coords_space_time
         shape = self.space_time_shape()
-        length = numpy.product(shape)
+        if len(shape) == 0:
+            length = 1
+            shape = (1,)
+        else:
+            length = numpy.product(shape)
 
         # Calculate the flattened shape
         new_shape = (length,)
@@ -551,6 +574,28 @@ class WaveSpectrum(DatasetWrapper):
     def peak_wave_speed(self) -> DataArray:
         return 2 * numpy.pi * self.peak_frequency() / self.peak_wavenumber
 
+    @property
+    def wavenumber_density(self) -> DataArray:
+        return self.variance_density * self.group_velocity / (numpy.pi * 2)
+
+    @property
+    def saturation_spectrum(self) -> DataArray:
+        return self.wavenumber_density * self.wavenumber**3
+
+    @property
+    def squared_slope_spectrum(self) -> DataArray:
+        return self.variance_density * self.wavenumber**2
+
+    def mean_squared_slope(self, fmin=0, fmax=numpy.inf) -> DataArray:
+        _range = self._range(fmin, fmax)
+
+        # Integrate dataset over frequencies. Make sure to fill any NaN entries with 0 before the integration.
+        return (
+            self.squared_slope_spectrum.fillna(0)
+            .isel({NAME_F: _range})
+            .integrate(coord=NAME_F)
+        )
+
     def m2(self, fmin=0, fmax=numpy.inf) -> DataArray:
         """
         Second order frequency moment. Primarily used in calculating the zero
@@ -603,7 +648,7 @@ class WaveSpectrum(DatasetWrapper):
         """
         return self.e.where(self._range(fmin, fmax), 0).argmax(dim=NAME_F)
 
-    def peak_frequency(self, fmin=0, fmax=numpy.inf) -> DataArray:
+    def peak_frequency(self, fmin=0, fmax=numpy.inf, use_spline=False) -> DataArray:
         """
         Peak frequency of the spectrum, i.e. frequency at which the spectrum
         obtains its maximum.
@@ -612,7 +657,14 @@ class WaveSpectrum(DatasetWrapper):
         :param fmax: maximum frequency
         :return: peak frequency
         """
-        return self.dataset[NAME_F][self.peak_index(fmin, fmax)]
+        if use_spline:
+            return DataArray(
+                data=spline_peak_frequency(self.frequency.values, self.e.values),
+                coords=self.coords_space_time,
+                dims=self.dims_space_time,
+            )
+        else:
+            return self.dataset[NAME_F][self.peak_index(fmin, fmax)]
 
     def peak_angular_frequency(self, fmin=0, fmax=numpy.inf) -> DataArray:
         """
@@ -625,7 +677,7 @@ class WaveSpectrum(DatasetWrapper):
         """
         return self.peak_frequency(fmin, fmax) * numpy.pi * 2
 
-    def peak_period(self, fmin=0, fmax=numpy.inf) -> DataArray:
+    def peak_period(self, fmin=0, fmax=numpy.inf, use_spline=False) -> DataArray:
         """
         Peak period of the spectrum, i.e. period at which the spectrum
         obtains its maximum.
@@ -634,9 +686,12 @@ class WaveSpectrum(DatasetWrapper):
         :param fmax: maximum frequency
         :return: peak period
         """
-        peak_period = 1 / self.peak_frequency(fmin, fmax)
+        peak_period = 1 / self.peak_frequency(fmin, fmax, use_spline=use_spline)
         peak_period.name = "peak period"
-        peak_period = peak_period.drop("frequency")
+        try:
+            peak_period = peak_period.drop("frequency")
+        except Exception:
+            pass
         return peak_period
 
     def peak_direction(self, fmin=0, fmax=numpy.inf) -> DataArray:
@@ -797,10 +852,25 @@ class WaveSpectrum(DatasetWrapper):
     def zero_crossing_period(self) -> DataArray:
         return self.tm02()
 
-    def interpolate(self: _T, coordinates) -> _T:
-        return self.__class__(interpolate_dataset_grid(coordinates, self.dataset))
+    def interpolate(self: _T, coordinates, extrapolation_value=0.0) -> _T:
+        object = self.__class__(interpolate_dataset_grid(coordinates, self.dataset))
+        object.fillna(extrapolation_value)
+        return object
 
-    def extrapolate_tail(self, end_frequency, power=-5) -> "FrequencySpectrum":
+    def extrapolate_tail(
+        self,
+        end_frequency,
+        power=None,
+        tail_energy=None,
+        tail_bounds=None,
+        tail_moments=None,
+    ) -> "FrequencySpectrum":
+        """
+        Extrapolate the tail using the given power
+        :param end_frequency: frequency to extrapolate to
+        :param power: power to use. If None, a best fit -4 or -5 tail is used.
+        :return:
+        """
         e = self.e
         a1 = self.a1
         b1 = self.b1
@@ -817,32 +887,57 @@ class WaveSpectrum(DatasetWrapper):
         tail_frequency = DataArray(
             data=tail_frequency, coords={"frequency": tail_frequency}, dims="frequency"
         )
-        tail_power = e.isel(frequency=-1) * (tail_frequency / frequency[-1]) ** power
-        tail_a1 = a1.isel(frequency=-1) * ones_like(tail_frequency)
-        tail_b1 = b1.isel(frequency=-1) * ones_like(tail_frequency)
-        tail_a2 = a2.isel(frequency=-1) * ones_like(tail_frequency)
-        tail_b2 = b2.isel(frequency=-1) * ones_like(tail_frequency)
-
-        e = concat((e, tail_power), dim="frequency")
-        a1 = concat((a1, tail_a1), dim="frequency")
-        b1 = concat((b1, tail_b1), dim="frequency")
-        a2 = concat((a2, tail_a2), dim="frequency")
-        b2 = concat((b2, tail_b2), dim="frequency")
-
-        return create_1d_spectrum(
-            e.frequency.values,
-            e.values,
-            e.time.values,
-            self.latitude.values,
-            self.longitude.values,
-            a1.values,
-            b1.values,
-            a2.values,
-            b2.values,
-            depth=self.depth.values,
-            dims=self.dims_space_time + [NAME_F],
+        variance_density = concat(
+            (e, e.isel(frequency=-1) * zeros_like(tail_frequency)), dim="frequency"
         )
 
+        tail_a1 = a1.isel(frequency=-1) if tail_moments is None else tail_moments["a1"]
+        tail_b1 = b1.isel(frequency=-1) if tail_moments is None else tail_moments["b1"]
+        tail_a2 = a2.isel(frequency=-1) if tail_moments is None else tail_moments["a2"]
+        tail_b2 = b2.isel(frequency=-1) if tail_moments is None else tail_moments["b2"]
+
+        a1 = concat((a1, tail_a1 * ones_like(tail_frequency)), dim="frequency")
+        b1 = concat((b1, tail_b1 * ones_like(tail_frequency)), dim="frequency")
+        a2 = concat((a2, tail_a2 * ones_like(tail_frequency)), dim="frequency")
+        b2 = concat((b2, tail_b2 * ones_like(tail_frequency)), dim="frequency")
+
+        if tail_energy is not None:
+            if isinstance(tail_energy, DataArray):
+                tail_energy = tail_energy.values
+
+            tail_information = (tail_bounds, tail_energy)
+        else:
+            tail_information = None
+
+        variance_density = DataArray(
+            data=numba_fill_zeros_or_nan_in_tail(
+                variance_density.values,
+                variance_density.frequency.values,
+                power,
+                tail_information=tail_information,
+            ),
+            dims=a1.dims,
+            coords=a1.coords,
+        )
+
+        dataset = Dataset(
+            {
+                "variance_density": variance_density,
+                "a1": a1,
+                "b1": b1,
+                "a2": a2,
+                "b2": b2,
+            }
+        )
+
+        for name in self.dataset:
+            if name in SPECTRAL_VARS:
+                continue
+            else:
+                dataset = dataset.assign({name: self.dataset[name]})
+
+        return FrequencySpectrum(dataset)
+
     def bandpass(self: _T, fmin=0, fmax=numpy.inf) -> _T:
 
         dataset = Dataset()
@@ -850,7 +945,7 @@ class WaveSpectrum(DatasetWrapper):
         for name in self.dataset:
             if name in SPECTRAL_VARS:
                 data = self.dataset[name].where(
-                    (self.frequency > fmin) & (self.frequency < fmax), drop=True
+                    (self.frequency >= fmin) & (self.frequency < fmax), drop=True
                 )
                 dataset = dataset.assign({name: data})
             else:
@@ -858,14 +953,14 @@ class WaveSpectrum(DatasetWrapper):
         cls = type(self)
         return cls(dataset)
 
-    def interpolate_frequency(
-        self: _T, new_frequencies, extrapolate_tail=True, extrapolate_power=5
-    ) -> _T:
-        return self.__class__(
+    def interpolate_frequency(self: _T, new_frequencies, extrapolation_value=0.0) -> _T:
+        object = self.__class__(
             interpolate_dataset_along_axis(
                 new_frequencies, self.dataset, coordinate_name="frequency"
             )
         )
+        object.fillna(extrapolation_value)
+        return object
 
     def _range(self, fmin=0.0, fmax=numpy.inf) -> numpy.ndarray:
         return (self.dataset[NAME_F].values >= fmin) & (
@@ -1028,6 +1123,29 @@ class FrequencyDirectionSpectrum(WaveSpectrum):
         )
         return self.as_frequency_spectrum()
 
+    def differentiate(self, coordinate=None, **kwargs) -> "FrequencyDirectionSpectrum":
+
+        if coordinate is None:
+            coordinate = "time"
+
+        if coordinate not in self.dataset:
+            raise ValueError(f"Coordinate {coordinate} does not exist in the dataset")
+
+        data = {
+            NAME_E: (
+                self.dims,
+                self.variance_density.differentiate(
+                    coordinate, datetime_unit="s", **kwargs
+                ).values,
+            )
+        }
+        for x in self.dataset:
+            if x in SPECTRAL_VARS:
+                continue
+            data[x] = (self.dims_space_time, self.dataset[x].values)
+
+        return FrequencyDirectionSpectrum(Dataset(data_vars=data, coords=self.coords()))
+
     @property
     def number_of_directions(self) -> int:
         return len(self.direction)
@@ -1046,7 +1164,27 @@ class FrequencySpectrum(WaveSpectrum):
     def __len__(self):
         return int(numpy.prod(self.spectral_values.shape[:-1]))
 
-    def interpolate(self: "FrequencySpectrum", coordinates) -> "FrequencySpectrum":
+    def interpolate_frequency(
+        self: "FrequencySpectrum",
+        new_frequencies,
+        extrapolation_value=0.0,
+        cumulative=False,
+    ) -> "FrequencySpectrum":
+        if cumulative:
+            interpolated_data = cumulative_frequency_interpolation_1d_variable(
+                new_frequencies, self.dataset
+            )
+            object = FrequencySpectrum(interpolated_data)
+            object.fillna(extrapolation_value)
+            return object
+        else:
+            return self.interpolate(
+                {NAME_F: new_frequencies}, extrapolation_value=extrapolation_value
+            )
+
+    def interpolate(
+        self: "FrequencySpectrum", coordinates, extrapolation_value=0.0
+    ) -> "FrequencySpectrum":
         """
 
         :param coordinates:
@@ -1071,7 +1209,9 @@ class FrequencySpectrum(WaveSpectrum):
                 interpolated_data[name] / interpolated_data[NAME_E]
             )
 
-        return FrequencySpectrum(interpolated_data)
+        object = FrequencySpectrum(interpolated_data)
+        object.fillna(extrapolation_value)
+        return object
 
     def as_frequency_direction_spectrum(
         self,
@@ -1235,3 +1375,133 @@ def load_spectrum_from_netcdf(
         return FrequencyDirectionSpectrum(dataset=dataset)
     else:
         return FrequencySpectrum(dataset=dataset)
+
+
+def fill_zeros_or_nan_in_tail(
+    spectrum: WaveSpectrum,
+    power=None,
+    tail_energy=None,
+    tail_bounds=None,
+) -> FrequencySpectrum:
+    variance_density = spectrum.e
+    a1 = spectrum.a1
+    b1 = spectrum.b1
+    a2 = spectrum.a2
+    b2 = spectrum.b2
+
+    if tail_energy is not None:
+        if isinstance(tail_energy, DataArray):
+            tail_energy = tail_energy.values
+
+        tail_information = (tail_bounds, tail_energy)
+    else:
+        tail_information = None
+
+    variance_density = DataArray(
+        data=numba_fill_zeros_or_nan_in_tail(
+            variance_density.values,
+            variance_density.frequency.values,
+            power,
+            tail_information=tail_information,
+        ),
+        dims=a1.dims,
+        coords=a1.coords,
+    )
+
+    dataset = Dataset(
+        {
+            "variance_density": variance_density,
+            "a1": a1,
+            "b1": b1,
+            "a2": a2,
+            "b2": b2,
+        }
+    )
+
+    for name in spectrum.dataset:
+        if name in SPECTRAL_VARS:
+            continue
+        else:
+            dataset = dataset.assign({name: spectrum.dataset[name]})
+
+    return FrequencySpectrum(dataset)
+
+
+def cumulative_frequency_interpolation_1d_variable(
+    interpolation_frequency, dataset: Dataset
+):
+    """
+    To interpolate the spectrum we first calculate a cumulative density function from the spectrum (which is essentialy
+    a pdf). We then interpolate the CDF function with a spline and differentiate the result.
+
+    :param interpolation_frequency:
+    :param dataset:
+    :return:
+    """
+
+    _dataset = Dataset()
+
+    # Copy over all non spectral vars
+    for name in dataset:
+        _name = str(name)
+        if _name not in SPECTRAL_VARS:
+            _dataset = _dataset.assign({_name: dataset[_name]})
+
+    coords = {
+        str(_coor_name): dataset[str(_coor_name)]
+        for _coor_name in dataset[NAME_E].coords
+    }
+    coords[NAME_F] = interpolation_frequency
+    dims = dataset[NAME_E].dims
+
+    # Interpolate energy
+    interpolated_energy = _cdf_interpolate(
+        interpolation_frequency,
+        dataset[NAME_F].values,
+        dataset[NAME_E].values,
+        interpolating_spline_order=3,
+        positive=True,
+    )
+
+    _dataset = _dataset.assign(
+        {
+            NAME_E: DataArray(
+                data=interpolated_energy,
+                coords=coords,
+                dims=dims,
+            )
+        }
+    )
+
+    # Interpolate scaling energy, to be used to renormalize moments.
+    interpolated_energy = _cdf_interpolate(
+        interpolation_frequency,
+        dataset[NAME_F].values,
+        dataset[NAME_E].values,
+        interpolating_spline_order=1,
+        positive=False,
+    )
+
+    for _name in SPECTRAL_MOMENTS:
+        interpolated_densities = (
+            _cdf_interpolate(
+                interpolation_frequency,
+                dataset[NAME_F].values,
+                dataset[_name].values * dataset[NAME_E].values,
+                interpolating_spline_order=1,
+                positive=False,
+            )
+            / interpolated_energy
+        )
+
+        _dataset = _dataset.assign(
+            {
+                _name: DataArray(
+                    data=interpolated_densities,
+                    coords=coords,
+                    dims=dims,
+                )
+            }
+        )
+
+    return _dataset
